@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2004 Pontus Fuchs, Giridhar Pemmasani
+ *  Copyright (C) 2003-2005 Pontus Fuchs, Giridhar Pemmasani
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,6 +17,8 @@
 #include <asm/uaccess.h>
 
 #include "ndis.h"
+#include "iw_ndis.h"
+#include "wrapper.h"
 
 #define MAX_PROC_STR_LEN 32
 
@@ -29,23 +31,22 @@ static int procfs_read_stats(char *page, char **start, off_t off,
 	char *p = page;
 	struct ndis_handle *handle = (struct ndis_handle *) data;
 	struct ndis_wireless_stats stats;
-	unsigned int res, written, needed;
-	long rssi;
+	NDIS_STATUS res;
+	ndis_rssi rssi;
 
 	if (off != 0) {
 		*eof = 1;
 		return 0;
 	}
 
-	res = doquery(handle, NDIS_OID_RSSI, (char*)&rssi,
-		      sizeof(rssi), &written, &needed);
+	res = miniport_query_info(handle, OID_802_11_RSSI, &rssi,
+				  sizeof(rssi));
 	if (!res)
-		p += sprintf(p, "signal_level=%ld dBm\n", rssi);
+		p += sprintf(p, "signal_level=%d dBm\n", (s32)rssi);
 
-	res = doquery(handle, NDIS_OID_STATISTICS, (char*)&stats,
-		      sizeof(stats), &written, &needed);
-	if (!res)
-	{
+	res = miniport_query_info(handle, OID_802_11_STATISTICS,
+				  &stats, sizeof(stats));
+	if (!res) {
 
 		p += sprintf(p, "tx_frames=%Lu\n", stats.tx_frag);
 		p += sprintf(p, "tx_multicast_frames=%Lu\n",
@@ -63,9 +64,9 @@ static int procfs_read_stats(char *page, char **start, off_t off,
 		p += sprintf(p, "fcs_errors=%Lu\n", stats.fcs_err);
 	}
 
-	if (p - page > count)
-	{
-		ERROR("wrote %u bytes (limit is %u)\n", p - page, count);
+	if (p - page > count) {
+		ERROR("wrote %lu bytes (limit is %u)\n",
+		      (unsigned long)(p - page), count);
 		*eof = 1;
 	}
 
@@ -77,8 +78,8 @@ static int procfs_read_encr(char *page, char **start, off_t off,
 {
 	char *p = page;
 	struct ndis_handle *handle = (struct ndis_handle *) data;
-	int i, encr_status, auth_mode, op_mode;
-	unsigned int res, written, needed;
+	int i, encr_status, auth_mode, infra_mode;
+	NDIS_STATUS res;
 	struct ndis_essid essid;
 	mac_address ap_address;
 
@@ -87,8 +88,8 @@ static int procfs_read_encr(char *page, char **start, off_t off,
 		return 0;
 	}
 
-	res = doquery(handle, NDIS_OID_BSSID, (char*)&ap_address,
-		      sizeof(ap_address), &written, &needed);
+	res = miniport_query_info(handle, OID_802_11_BSSID,
+				  &ap_address, sizeof(ap_address));
 	if (res)
 		memset(ap_address, 0, ETH_ALEN);
 	p += sprintf(p, "ap_address=%2.2X", ap_address[0]);
@@ -96,48 +97,44 @@ static int procfs_read_encr(char *page, char **start, off_t off,
 		p += sprintf(p, ":%2.2X", ap_address[i]);
 	p += sprintf(p, "\n");
 
-	res = doquery(handle, NDIS_OID_ESSID, (char*)&essid,
-		      sizeof(essid), &written, &needed);
-	if (!res)
-	{
+	res = miniport_query_info(handle, OID_802_11_SSID, &essid,
+				  sizeof(essid));
+	if (!res) {
 		essid.essid[essid.length] = '\0';
 		p += sprintf(p, "essid=%s\n", essid.essid);
 	}
 
-	res = query_int(handle, NDIS_OID_ENCR_STATUS, &encr_status);
-	res |= query_int(handle, NDIS_OID_AUTH_MODE, &auth_mode);
+	res = miniport_query_int(handle, OID_802_11_ENCRYPTION_STATUS,
+				 &encr_status);
+	res |= miniport_query_int(handle, OID_802_11_AUTHENTICATION_MODE,
+				  &auth_mode);
 
-	if (!res)
-	{
-		int active = handle->encr_info.active;
-		p += sprintf(p, "tx_key=%u\n", handle->encr_info.active);
+	if (!res) {
+		int t = handle->encr_info.tx_key_index;
+		p += sprintf(p, "tx_key=%u\n", handle->encr_info.tx_key_index);
 		p += sprintf(p, "key=");
-		if (handle->encr_info.keys[active].length > 0)
+		if (handle->encr_info.keys[t].length > 0)
 			for (i = 0; i < NDIS_ENCODING_TOKEN_MAX &&
-				     i < handle->encr_info.keys[active].length;
+				     i < handle->encr_info.keys[t].length;
 			     i++)
 				p += sprintf(p, "%2.2X",
-					     handle->encr_info.keys[active].key[i]);
+					     handle->encr_info.keys[t].key[i]);
 		else
 			p += sprintf(p, "off");
 		p += sprintf(p, "\n");
 
-		p += sprintf(p, "status=%sabled\n",
-			     (encr_status == ENCR_DISABLED) ? "dis" : "en");
-		p += sprintf(p, "auth_mode=%s\n",
-			     (auth_mode == AUTHMODE_AUTO) ?
-			     "auto" :
-			     (auth_mode == AUTHMODE_RESTRICTED) ?
-			     "restricted" : "open");
+		p += sprintf(p, "status=%d\n", encr_status);
+		p += sprintf(p, "auth_mode=%d\n", auth_mode);
 	}
 
-	res = query_int(handle, NDIS_OID_MODE, &op_mode);
-	p += sprintf(p, "mode=%s\n", (op_mode == NDIS_MODE_ADHOC) ?
-		     "adhoc" : (op_mode == NDIS_MODE_INFRA) ?
+	res = miniport_query_int(handle, OID_802_11_INFRASTRUCTURE_MODE,
+				 &infra_mode);
+	p += sprintf(p, "mode=%s\n", (infra_mode == Ndis802_11IBSS) ?
+		     "adhoc" : (infra_mode == Ndis802_11Infrastructure) ?
 		     "managed" : "auto");
-	if (p - page > count)
-	{
-		WARNING("wrote %u bytes (limit is %u)", p - page, count);
+	if (p - page > count) {
+		WARNING("wrote %lu bytes (limit is %u)",
+			(unsigned long)(p - page), count);
 		*eof = 1;
 	}
 
@@ -150,19 +147,22 @@ static int procfs_read_hw(char *page, char **start, off_t off,
 	char *p = page;
 	struct ndis_handle *handle = (struct ndis_handle *)data;
 	struct ndis_configuration config;
-	unsigned int res, written, needed, power_mode;
-	unsigned long tx_power, bit_rate, rts_threshold, frag_threshold;
-	unsigned long antenna;
+	unsigned int power_mode;
+	NDIS_STATUS res;
+	ndis_tx_power_level tx_power;
+	ULONG bit_rate;
+	ndis_rts_threshold rts_threshold;
+	ndis_fragmentation_threshold frag_threshold;
+	ndis_antenna antenna;
 
 	if (off != 0) {
 		*eof = 1;
 		return 0;
 	}
 
-	res = doquery(handle, NDIS_OID_CONFIGURATION,
-		      (char*)&config, sizeof(config), &written, &needed);
-	if (!res)
-	{
+	res = miniport_query_info(handle, OID_802_11_CONFIGURATION,
+				  &config, sizeof(config));
+	if (!res) {
 		p += sprintf(p, "beacon_period=%u msec\n",
 			     config.beacon_period);
 		p += sprintf(p, "atim_window=%u msec\n", config.atim_window);
@@ -175,27 +175,27 @@ static int procfs_read_hw(char *page, char **start, off_t off,
 			     config.fh_config.dwell_time);
 	}
 
-	res = doquery(handle, NDIS_OID_TX_POWER_LEVEL, (char*)&tx_power,
-		      sizeof(tx_power), &written, &needed);
+	res = miniport_query_info(handle, OID_802_11_TX_POWER_LEVEL,
+				  &tx_power, sizeof(tx_power));
 	if (!res)
-		p += sprintf(p, "tx_power=%lu mW\n", tx_power);
+		p += sprintf(p, "tx_power=%u mW\n", tx_power);
 
-	res = doquery(handle, NDIS_OID_GEN_SPEED, (char*)&bit_rate,
-		      sizeof(bit_rate), &written, &needed);
+	res = miniport_query_info(handle, OID_GEN_LINK_SPEED,
+				  &bit_rate, sizeof(bit_rate));
 	if (!res)
-		p += sprintf(p, "bit_rate=%lu kBps\n", bit_rate / 10);
+		p += sprintf(p, "bit_rate=%u kBps\n", (u32)bit_rate / 10);
 
-	res = doquery(handle, NDIS_OID_RTS_THRESH, (char*)&rts_threshold,
-		      sizeof(rts_threshold), &written, &needed);
+	res = miniport_query_info(handle, OID_802_11_RTS_THRESHOLD,
+				  &rts_threshold, sizeof(rts_threshold));
 	if (!res)
-		p += sprintf(p, "rts_threshold=%lu bytes\n", rts_threshold);
+		p += sprintf(p, "rts_threshold=%u bytes\n", rts_threshold);
 
-	res = doquery(handle, NDIS_OID_FRAG_THRESH, (char*)&frag_threshold,
-		      sizeof(frag_threshold), &written, &needed);
+	res = miniport_query_info(handle, OID_802_11_FRAGMENTATION_THRESHOLD,
+				  &frag_threshold, sizeof(frag_threshold));
 	if (!res)
-		p += sprintf(p, "frag_threshold=%lu bytes\n", frag_threshold);
+		p += sprintf(p, "frag_threshold=%u bytes\n", frag_threshold);
 
-	res = query_int(handle, NDIS_OID_POWER_MODE, &power_mode);
+	res = miniport_query_int(handle, OID_802_11_POWER_MODE, &power_mode);
 	if (!res)
 		p += sprintf(p, "power_mode=%s\n",
 			     (power_mode == NDIS_POWER_OFF) ?
@@ -203,27 +203,24 @@ static int procfs_read_hw(char *page, char **start, off_t off,
 			     (power_mode == NDIS_POWER_MAX) ?
 			     "max_savings" : "min_savings");
 
-	res = doquery(handle, NDIS_OID_NUM_ANTENNA, (char *)&antenna,
-		      sizeof(antenna), &written, &needed);
+	res = miniport_query_info(handle, OID_802_11_NUMBER_OF_ANTENNAS,
+				  &antenna, sizeof(antenna));
 	if (!res)
-		p += sprintf(p, "num_antennas=%lu\n",
-			     antenna);
+		p += sprintf(p, "num_antennas=%u\n", antenna);
 
-	res = doquery(handle, NDIS_OID_TX_ANTENNA, (char *)&antenna,
-		      sizeof(antenna), &written, &needed);
+	res = miniport_query_info(handle, OID_802_11_TX_ANTENNA_SELECTED,
+				  &antenna, sizeof(antenna));
 	if (!res)
-		p += sprintf(p, "tx_antenna=%lu\n",
-			     antenna);
+		p += sprintf(p, "tx_antenna=%u\n", antenna);
 
-	res = doquery(handle, NDIS_OID_RX_ANTENNA, (char *)&antenna,
-		      sizeof(antenna), &written, &needed);
+	res = miniport_query_info(handle, OID_802_11_RX_ANTENNA_SELECTED,
+				  &antenna, sizeof(antenna));
 	if (!res)
-		p += sprintf(p, "rx_antenna=%lu\n",
-			     antenna);
+		p += sprintf(p, "rx_antenna=%u\n", antenna);
 
-	if (p - page > count)
-	{
-		WARNING("wrote %u bytes (limit is %u)", p - page, count);
+	if (p - page > count) {
+		WARNING("wrote %lu bytes (limit is %u)",
+			(unsigned long)(p - page), count);
 		*eof = 1;
 	}
 
@@ -235,6 +232,7 @@ static int procfs_read_settings(char *page, char **start, off_t off,
 {
 	char *p = page;
 	struct ndis_handle *handle = (struct ndis_handle *)data;
+	struct device_setting *setting;
 
 	if (off != 0) {
 		*eof = 1;
@@ -244,6 +242,10 @@ static int procfs_read_settings(char *page, char **start, off_t off,
 	p += sprintf(p, "hangcheck_interval=%d\n",
 		     (int)(handle->hangcheck_interval / HZ));
 
+	list_for_each_entry(setting, &handle->device->settings, list) {
+		p += sprintf(p, "%s=%s\n", setting->name, setting->value);
+	}
+
 	return (p - page);
 }
 
@@ -266,8 +268,7 @@ static int procfs_write_settings(struct file *file, const char *buf,
 	if ((p = strchr(setting, '=')))
 		*p = 0;
 
-	if (!strcmp(setting, "hangcheck_interval"))
-	{
+	if (!strcmp(setting, "hangcheck_interval")) {
 		int i;
 
 		if (!p)
@@ -277,9 +278,7 @@ static int procfs_write_settings(struct file *file, const char *buf,
 		hangcheck_del(handle);
 		handle->hangcheck_interval = i * HZ;
 		hangcheck_add(handle);
-	}
-	else if (!strcmp(setting, "suspend"))
-	{
+	} else if (!strcmp(setting, "suspend")) {
 		int i;
 
 		if (!p)
@@ -288,25 +287,18 @@ static int procfs_write_settings(struct file *file, const char *buf,
 		i = simple_strtol(p, NULL, 10);
 		if (i <= 0 || i > 3)
 			return -EINVAL;
-		if (handle->device->bustype == 5)
-			ndis_suspend_pci(handle->dev.pci, i);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-		else
-			ndis_suspend_usb(handle->intf, i);
-#endif
-	}
-	else if (!strcmp(setting, "resume")) {
-		if (handle->device->bustype == 5)
-			ndis_resume_pci(handle->dev.pci);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-		else
-			ndis_resume_usb(handle->intf);
-#endif
-	} else if (!strcmp(setting, "power_profile"))
-	{
+		if (handle->device->bustype == NDIS_PCI_BUS)
+			ndiswrapper_suspend_pci(handle->dev.pci, i);
+	} else if (!strcmp(setting, "resume")) {
+		if (handle->device->bustype == NDIS_PCI_BUS)
+			ndiswrapper_resume_pci(handle->dev.pci);
+	} else if (!strcmp(setting, "reinit")) {
+		if (ndis_reinit(handle))
+			return -EINVAL;
+	} else if (!strcmp(setting, "power_profile")) {
 		int i;
 		struct miniport_char *miniport;
-		unsigned long profile_inf;
+		ULONG profile_inf;
 
 		if (!p)
 			return -EINVAL;
@@ -321,34 +313,70 @@ static int procfs_write_settings(struct file *file, const char *buf,
 
 		/* 1 for AC and 0 for Battery */
 		if (i)
-			profile_inf = NDIS_POWER_PROFILE_AC;
+			profile_inf = NdisPowerProfileAcOnLine;
 		else
-			profile_inf = NDIS_POWER_PROFILE_BATTERY;
+			profile_inf = NdisPowerProfileBattery;
 		
 		miniport->pnp_event_notify(handle->adapter_ctx,
-					   NDIS_PNP_PROFILE_CHANGED,
+					   NdisDevicePnPEventPowerProfileChanged,
 					   &profile_inf, sizeof(profile_inf));
+	} else if (!strcmp(setting, "auth_mode")) {
+		int i;
+
+		if (!p)
+			return -EINVAL;
+		p++;
+		i = simple_strtol(p, NULL, 10);
+		if (i <= 0 || i > 5)
+			return -EINVAL;
+
+		if (set_auth_mode(handle, i))
+			return -EINVAL;
+	} else if (!strcmp(setting, "encr_mode")) {
+		int i;
+
+		if (!p)
+			return -EINVAL;
+		p++;
+		i = simple_strtol(p, NULL, 10);
+		if (i <= 0 || i > 7)
+			return -EINVAL;
+
+		if (set_encr_mode(handle, i))
+			return -EINVAL;
+	} else {
+		int res = -1;
+		struct device_setting *dev_setting;
+
+		if (!p)
+			TRACEEXIT1(return -EINVAL);
+		p++;
+		DBGTRACE1("name='%s', value='%s'\n", setting, p);
+		list_for_each_entry(dev_setting, &handle->device->settings,
+				    list) {
+			struct ndis_config_param *param;
+
+			param = &dev_setting->config_param;
+			if (!stricmp(dev_setting->name, setting)) {
+				if (strlen(p) > MAX_NDIS_SETTING_VALUE_LEN)
+					TRACEEXIT1(return -EINVAL);
+				memset(dev_setting->value, 0,
+				       MAX_NDIS_SETTING_VALUE_LEN);
+				memcpy(dev_setting->value, p, strlen(p));
+				if (param->type == NDIS_CONFIG_PARAM_STRING)
+					RtlFreeUnicodeString(&param->data.ustring);
+				param->type = NDIS_CONFIG_PARAM_NONE;
+				res = 0;
+			}
+		}
+		if (res)
+			return -EINVAL;
 	}
-	else
-		return -EINVAL;
 
 	return count;
 
 }
 
-int ndiswrapper_procfs_init(void)
-{
-	ndiswrapper_procfs_entry = proc_mkdir(DRV_NAME, proc_net);
-	if (ndiswrapper_procfs_entry == NULL)
-	{
-		ERROR("%s", "Couldn't create procfs directory");
-		return -ENOMEM;
-	}
-	ndiswrapper_procfs_entry->uid = proc_uid;
-	ndiswrapper_procfs_entry->gid = proc_gid;
-	return 0;
-}
-
 int ndiswrapper_procfs_add_iface(struct ndis_handle *handle)
 {
 	struct net_device *dev = handle->net_dev;
@@ -362,8 +390,7 @@ int ndiswrapper_procfs_add_iface(struct ndis_handle *handle)
 
 	handle->procfs_iface = proc_iface;
 
-	if (proc_iface == NULL)
-	{
+	if (proc_iface == NULL) {
 		ERROR("%s", "Couldn't create proc directory");
 		return -ENOMEM;
 	}
@@ -372,13 +399,10 @@ int ndiswrapper_procfs_add_iface(struct ndis_handle *handle)
 
 	procfs_entry = create_proc_entry("hw", S_IFREG | S_IRUSR | S_IRGRP,
 					 proc_iface);
-	if (procfs_entry == NULL)
-	{
+	if (procfs_entry == NULL) {
 		ERROR("%s", "Couldn't create proc entry for 'hw'");
 		return -ENOMEM;
-	}
-	else
-	{
+	} else {
 		procfs_entry->uid = proc_uid;
 		procfs_entry->gid = proc_gid;
 		procfs_entry->data = handle;
@@ -387,13 +411,10 @@ int ndiswrapper_procfs_add_iface(struct ndis_handle *handle)
 
 	procfs_entry = create_proc_entry("stats", S_IFREG | S_IRUSR | S_IRGRP,
 					 proc_iface);
-	if (procfs_entry == NULL)
-	{
+	if (procfs_entry == NULL) {
 		ERROR("%s", "Couldn't create proc entry for 'stats'");
 		return -ENOMEM;
-	}
-	else
-	{
+	} else {
 		procfs_entry->uid = proc_uid;
 		procfs_entry->gid = proc_gid;
 		procfs_entry->data = handle;
@@ -402,13 +423,10 @@ int ndiswrapper_procfs_add_iface(struct ndis_handle *handle)
 
 	procfs_entry = create_proc_entry("encr", S_IFREG | S_IRUSR | S_IRGRP,
 					 proc_iface);
-	if (procfs_entry == NULL)
-	{
+	if (procfs_entry == NULL) {
 		ERROR("%s", "Couldn't create proc entry for 'encr'");
 		return -ENOMEM;
-	}
-	else
-	{
+	} else {
 		procfs_entry->uid = proc_uid;
 		procfs_entry->gid = proc_gid;
 		procfs_entry->data = handle;
@@ -418,13 +436,10 @@ int ndiswrapper_procfs_add_iface(struct ndis_handle *handle)
 	procfs_entry = create_proc_entry("settings", S_IFREG |
 					 S_IRUSR | S_IRGRP |
 					 S_IWUSR | S_IWGRP, proc_iface);
-	if (procfs_entry == NULL)
-	{
-		ERROR("%s", "Couldn't create proc entry for 'encr'");
+	if (procfs_entry == NULL) {
+		ERROR("%s", "Couldn't create proc entry for 'settings'");
 		return -ENOMEM;
-	}
-	else
-	{
+	} else {
 		procfs_entry->uid = proc_uid;
 		procfs_entry->gid = proc_gid;
 		procfs_entry->data = handle;
@@ -450,9 +465,77 @@ void ndiswrapper_procfs_remove_iface(struct ndis_handle *handle)
 	handle->procfs_iface = NULL;
 }
 
+static int procfs_read_debug(char *page, char **start, off_t off,
+			     int count, int *eof, void *data)
+{
+	char *p = page;
+
+	if (off != 0) {
+		*eof = 1;
+		return 0;
+	}
+
+	p += sprintf(p, "%d\n", debug);
+
+	return (p - page);
+}
+
+static int procfs_write_debug(struct file *file, const char *buf,
+			      unsigned long count, void *data)
+{
+	char debug_level[MAX_PROC_STR_LEN];
+	int i;
+
+	if (count > MAX_PROC_STR_LEN)
+		return -EINVAL;
+
+	memset(debug_level, 0, sizeof(debug_level));
+	if (copy_from_user(debug_level, buf, count))
+		return -EFAULT;
+
+	i = simple_strtol(debug_level, NULL, 10);
+#if defined(DEBUG) && DEBUG > 0
+	if (i < 0 || i > DEBUG)
+		return -EINVAL;
+#else
+	return -EINVAL;
+#endif
+	debug = i;
+
+	return count;
+}
+
+int ndiswrapper_procfs_init(void)
+{
+	struct proc_dir_entry *procfs_entry;
+
+	ndiswrapper_procfs_entry = proc_mkdir(DRIVER_NAME, proc_net);
+	if (ndiswrapper_procfs_entry == NULL) {
+		ERROR("couldn't create procfs directory");
+		return -ENOMEM;
+	}
+	ndiswrapper_procfs_entry->uid = proc_uid;
+	ndiswrapper_procfs_entry->gid = proc_gid;
+
+	procfs_entry = create_proc_entry("debug", S_IFREG | S_IRUSR | S_IRGRP,
+					 ndiswrapper_procfs_entry);
+	if (procfs_entry == NULL) {
+		ERROR("couldn't create proc entry for 'debug'");
+		return -ENOMEM;
+	} else {
+		procfs_entry->uid = proc_uid;
+		procfs_entry->gid = proc_gid;
+		procfs_entry->read_proc  = procfs_read_debug;
+		procfs_entry->write_proc = procfs_write_debug;
+	}
+
+	return 0;
+}
+
 void ndiswrapper_procfs_remove(void)
 {
 	if (ndiswrapper_procfs_entry == NULL)
 		return;
-	remove_proc_entry(DRV_NAME, proc_net);
+	remove_proc_entry("debug", ndiswrapper_procfs_entry);
+	remove_proc_entry(DRIVER_NAME, proc_net);
 }
